@@ -3,14 +3,15 @@ package LUGS::Events::Parser;
 use strict;
 use warnings;
 use base qw(LUGS::Events::Parser::Filter);
-use boolean qw(true);
+use boolean qw(true false);
 
 use Carp qw(croak);
-use Data::Schema qw(ds_validate);
+use DateTime ();
+use List::MoreUtils qw(all);
 use LUGS::Events::Parser::Event ();
 use Params::Validate ':all';
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 validation_options(
     on_fail => sub
@@ -48,19 +49,29 @@ sub _init
         my $valid_handlers = sub
         {
             my ($data) = @_;
-            my $handler = [ 'array', { min_len => 1, of => [ 'hash', {
-                required_keys => [ qw(rewrite fields) ],
-                keys => {
-                    rewrite => [ 'str' ],
-                    fields  => [ 'array', { min_len => 1 } ],
-                },
-            } ] } ];
-            my @keys = keys %$data;
-            my $schema = [ 'hash', {
-                allowed_keys => [ @keys ],
-                keys => { map { $_ => $handler } @keys },
-            } ];
-            return ds_validate($data, $schema)->{success};
+
+            return false unless ref $data eq 'HASH';
+
+            foreach my $tagname (keys %$data) {
+                return false unless ref $data->{$tagname} eq 'ARRAY';
+                return false unless scalar @{$data->{$tagname}};
+
+                foreach my $entry (@{$data->{$tagname}}) {
+                    return false unless ref $entry eq 'HASH';
+
+                    my %keys = map { $_ => true } keys %$entry;
+
+                    return false unless scalar keys %keys == 2;
+                    return false unless all { exists $keys{$_} } qw(rewrite fields);
+
+                    return false unless ref \$entry->{rewrite} eq 'SCALAR';
+                    return false unless ref  $entry->{fields}  eq 'ARRAY';
+
+                    return false unless scalar @{$entry->{fields}};
+                }
+            }
+
+            return true;
         };
 
         my @args = %$opts;
@@ -122,7 +133,7 @@ sub _parse_content
                 last;
             }
             else {
-                my ($name, $text) = $field =~ /^\s+ (\w+?) \s+ (.+)/x;
+                my ($name, $text) = $field =~ /^\s+ (\w+?) \s+ (.*)/x;
                 if ($self->{Filter_html}) {
                     my @html;
                     $self->_parse_html($text, \@html);
@@ -141,6 +152,14 @@ sub _parse_content
             $self->_decode_entities(\%fields);
         }
 
+        my ($year, $month, $day) = $fields{event} =~ /^(\d{4})(\d{2})(\d{2})$/;
+        my $dt = DateTime->new(year => $year, month => $month, day => $day);
+        my $i = 1;
+        my %weekdays = map { $i++ => $_ } qw(Mo Di Mi Do Fr Sa So);
+
+        $fields{day}     ||= $1 if $day =~ /^0?(.+)$/;
+        $fields{weekday} ||= $weekdays{$dt->day_of_week};
+
         my ($event, $color) = map $fields{$_}, qw(event color);
         my $id = $ids{$event}->{$color}++;
         $fields{anchor} = join '_', ($event, $id, $color);
